function hdr

lambda = 10;

P = 13;
inputs = cell(P,1);
for i = 1:P
    inputs{i} = sprintf('img%02d.jpg',i);
end
exps = [13 10 4 3.2 1 0.8 0.3 1/4 1/60 1/80 1/320 1/400 1/1000];
I = cell(P,1);
for j = 1:P
    I{j} = imread(inputs{j});
end

% Get the subset of pixels that we are working on
load points.mat x y N

% Z contains all 3 RGB channels
Z = zeros(N,P,3);
for j = 1:P
    ind = sub2ind(size(I{j}),y,x);
    for c = 1:3
        channel = I{j}(:,:,c);
        Z(:,j,c) = reshape(channel(ind),[N 1]);
    end
end

g = zeros(256,3);
logE = zeros(N,3);

% Do each color channel independently
for c = 1:3
    A = sparse(N*P+255,N+256);
    b = sparse(N*P+255,1);
    
    k = 0;
    for j = 1:P
        t = log(exps(j));
        for i = 1:N
            k = k+1;
            w = weight(Z(i,j,c));
            A(k,Z(i,j,c)+1) = 1;
            A(k,256+i) = -w;
            b(k) = w*t;
        end
    end
    
    % Response curve smoothness constraints
    k = N*P;
    for i = 1:254
        k = k+1;
        A(k,i) = lambda;
        A(k,i+1) = -2*lambda;
        A(k,i+2) = lambda;
    end
    
    % g(Zmid) = 0 constraint
    A(size(A,1),128) = 1;
    
    % Solve system
    x = A \ b;
    
    g(:,c) = x(1:256)./weight((0:255)');
    logE(:,c) = x((1:N)+256);
end

figure;
plot(0:255,g(:,1),'r',0:255,g(:,2),'g',0:255,g(:,3),'b')

% Reconstruct radiance map
[m,n,c] = size(I{1});
num = zeros(m,n,3);
den = zeros(m,n,3);
for c = 1:3
    for j = 1:P
        Z = I{j}(:,:,c);
        w = weight(Z);
        num(:,:,c) = num(:,:,c) + w.*(reshape(g(Z+1,c),size(Z))-log(exps(j)));
        den(:,:,c) = den(:,:,c) + w;
    end
end
E = exp(num./den);
rgb = tonemap(E,'AdjustLightness',[0.1 1],'AdjustSaturation',3);
figure;
imshow(rgb);
% imwrite(rgb,'tonemapped.jpg');


function w = weight(z)
w = double(z)+0.4;
w(z > 127) = 256 - z(z > 127);
